from nptdms import TdmsFile


def loadTdms(
    fileName,
    group='data',
    accelMps=[1, 2, 3, 4, 5, 6, 7, 8, 9],
    dispMps=[1, 2, 3, 4],
):
    """加载tdms文件

    Args:
        fileName (str): tdms文件路径
        group (str, optional): tdms文件中所要加载的分组. Defaults to 'data'.
        accelMps (list, optional): 所使用的加速度测点. Defaults to [1, 2, 3, 4, 5, 6, 7, 8, 9].
        dispMps (list, optional): 所使用的位移测点. Defaults to [1, 2].

    Returns:
        tuple(accelMps, dispMps, accelDict, dispDict): 加速度字典和位移字典，key -> 测点号，val -> 测点时程数组

    初始化时被调用，也可作为静态方法单独使用
    """

    chAccelMap = {  # 加速度通道
        '1': '加速度0',
        '2': '加速度1',
        '3': '加速度2',
        '4': '加速度3',
        '5': '加速度4',
        '6': '加速度5',
        '7': '加速度6',
        '8': '加速度7',
        '9': '加速度8'
    }

    chDispMap = {  # 位移通道
        '1': '位移0',
        '2': '位移1',
        '3': '位移2',
        '4': '位移3'

    }

    with TdmsFile.open(fileName) as tdmsFile:  # 加载文件
        group = tdmsFile[group]  # 获取组

        # 提取加速度数据
        accelSet = {}
        for mp in accelMps:
            if mp in [2, 5, 8]:
                accelSet[mp] = group[chAccelMap[str(
                    mp)]][:] * (-1) * 10 * 9.8
            else:
                accelSet[mp] = group[chAccelMap[str(mp)]][:] * 10 * 9.8
        # 位移
        dispSet = {}
        for mp in dispMps:
            dispSet[mp] = group[chDispMap[str(mp)]][:] / (-100)

    return accelMps, dispMps,accelSet, dispSet



import numpy as np
from wwprocess.calculation.process import Process


class Rms(Process):
    """
    传入模态试验对象，计算所有测试轮次的位移和加速度的rms
    """

    def __init__(self, exp):
        """
        计算所有风速/模态测试轮次的下所有测点的加速度和位移均方根

        Args:
            exp (Wt/Mt): 传入试验对象

        """
        self.accelDataSet, self.dispDataSet = self.process(exp)

    def _Process__calc(self, mp):
        """
        传入测点的时程曲线，计算均方根

        Args:
            mp (_type_): _description_
        """
        return np.std(mp)


class Avg(Process):
    """
    传入模态试验对象，计算所有测试轮次的位移和加速度的rms
    """

    def __init__(self, exp):
        """
        计算所有风速/模态测试轮次的下所有测点的加速度和位移均方根

        Args:
            exp (Wt/Mt): 传入试验对象

        """
        self.accelDataSet, self.dispDataSet = self.process(exp)

    def _Process__calc(self, mp):
        """
        传入测点的时程曲线，计算均值

        Args:
            mp (_type_): _description_
        """
        return np.mean(mp)



from wwprocess.calculation.process import Process, ProcessCsd
import numpy as np
from scipy import signal


def anpsdCalc(psdInOneRound,
              mps):
    """
    输入一次试验的psd字典：key -> 测点号，val -> psd数组，以及使用到的测点号，返回anpsd数组
    """

    f = psdInOneRound[1][0]
    rowSize = f.size
    colSize = len(mps)
    normPsds = np.empty([rowSize, colSize], dtype=float)
    for mp in psdInOneRound:
        if mp not in mps:
            continue
        normPsds[:, int(mp) - 1] = psdInOneRound[mp][1] / np.sum(psdInOneRound[mp][1])
    anpsdValue = np.mean(normPsds, axis=1)
    return f, anpsdValue


def anpsd(
        psdModal,
        accelMps=[1, 2, 3, 4, 5, 6, 7, 8, 9],
        dispMps=[1, 2]):
    # 输出：字典，键为试验轮次，值为tuple(f, anpsd)
    accelAnpsd = {}
    for round in psdModal.accelData:
        accelAnpsd[round] = [None, None]
        accelAnpsd[round][0], accelAnpsd[round][1] = anpsdCalc(psdModal.accelData[round], mps=accelMps)

    dispAnpsd = {}
    for round in psdModal.dispData:
        dispAnpsd[round] = [None, None]
        dispAnpsd[round][0], dispAnpsd[round][1] = anpsdCalc(psdModal.dispData[round], mps=dispMps)

    psdModal.accelAnpsd = accelAnpsd
    psdModal.dispAnpsd = dispAnpsd

class Psd(Process):
    """
    计算psd

    数据为2级字典，第一级为风速/模态测试轮次，第二级为测点号，值为tuple，第一个元素为f，第二个为amp
    """

    def __init__(
        self,
        exp,
        nfft=2 ** 12,  # 变换点数
        fs=500,  # 采样频率
        window='boxcar',
        nperseg='',
        detrend='linear',
        noverlap=0,
    ):
        self.nfft = nfft
        self.fs = fs
        self.window = window
        if nperseg:
            self.nperseg = nperseg
        else:
            self.nperseg = nfft / 4
        self.detrend = detrend
        self.noverlap = noverlap

        self.accelData, self.dispData = self.process(exp)

    def _Process__calc(self, mp):

        # 计算功率谱
        return signal.welch(
            mp,
            fs=self.fs,
            window=self.window,
            nfft=self.nfft,
            nperseg=self.nperseg,
            detrend=self.detrend,
            noverlap=self.noverlap,
        )


class Csd(ProcessCsd):
    """
    计算csd

    数据为2级字典，第一级为风速/模态测试轮次，第二级为测点号，值为tuple，第一个元素为f，第二个为amp
    """

    def __init__(
        self,
        exp,
        accelRef=3,
        dispRef=2,
        nfft=2 ** 12,  # 变换点数
        fs=500,  # 采样频率
        window='boxcar',
        nperseg='',
        detrend='linear',
        noverlap=0,
    ):
        self.accelRefData = accelRef
        self.dispRef = dispRef
        self.nfft = nfft
        self.fs = fs
        self.window = window
        if nperseg:
            self.nperseg = nperseg
        else:
            self.nperseg = nfft / 4
        self.detrend = detrend
        self.noverlap = noverlap

        self.accelData, self.dispData = self.processCsd(exp, accelRef, dispRef)

    def _ProcessCsd__calcCsd(self, mp, mpRef):

        # 计算功率谱
        return signal.csd(
            mp,
            mpRef,
            fs=self.fs,
            window=self.window,
            nfft=self.nfft,
            nperseg=self.nperseg,
            detrend=self.detrend,
            noverlap=self.noverlap,
        )



import numpy as np
from scipy.interpolate import interp1d
from wwprocess.dataset.wt import Wt
from wwprocess.plots.PlotSchema import PlotSchema


def Densify(wt, density=10):
    """
    将wt中的每个数据进行插值加密，两点之间的插值个数为density
    """
    for ws in wt.windSpeedSeq:
        for mp in wt.accelMps:
            wt.accelDataSet[ws][mp] = dens(wt.accelDataSet[ws][mp], density=density)
        for mp in wt.dispMps:
            wt.dispDataSet[ws][mp] = dens(wt.dispDataSet[ws][mp], density=density)

def dens(
    data, 
    density = 10 # 每两个点之间的点数
):
    dataLen = len(data)
    # 横轴
    x = np.arange(0, dataLen)
    # 插值函数
    f1 = interp1d(x, data, kind='linear')
    # 确定加密后的点个数
    # 公式：(点数 - 1) * 两点之间插值个数 * 点数
    # 例如，有20个点，每两个点之间插10个值，则共有(20 -1) * 10 + 20 = 210个点
    totalPoints = (dataLen - 1) * density + dataLen
    denseSeq = np.linspace(0, dataLen - 1, totalPoints)
    return f1(denseSeq)



import os
from wwprocess.dataset.loadTdms import loadTdms

"""
将raw目录下的一个模态试验对应的一个文件夹封装为一个对象

属性包含：
* 模态试验下各个测点的加速度和位移时程数据
方法：
* 在初始化时加载加速度和位移数据
"""


class Mt:  # modalTest
    """封装了风洞试验数据的对象

    Args:
        shape(str): 截面形状 
        testDir(str, optional): 可指定试验文件夹名称
        noGlued(bool, optional): 是否未贴胶带
    """

    def __init__(self,
                 shape,  # 形状 ori rect ..
                 testDir='',  # 输入模态测试的文件夹名，为空则使用默认命名方式
                 noGlued=False,  # 是否粘住缝隙
                 ):
        # 空值
        self.dispMps = []  # 使用的位移测点
        self.accelMps = []  # 使用的加速度测点
        self.accelDataSet = {}  # 二级字典，第一级为第几轮测试，第二级为测点号
        self.dispDataSet = {}  # 位移，逻辑同上
        # 描述配置
        # 初始化的值
        self.shape = shape
        self.testDir = testDir
        self.noGlued = noGlued
        # TODO
        self.__loadDataInAllRounds()

    # 计算所有风速的均方根，二级词典，一级为测点，二级为风速
    # 可传入使用的测点编号
    def __loadDataInAllRounds(self):
        """
        加载模态测试数据

        在初始化时被调用，更新加速度和位移属性
        """
        # 处理文件路径
        if self.testDir:
            dirPath = f'raw/{self.testDir}'
        else:
            if self.noGlued:
                dirPath = f'raw/{self.shape}_modalTest_noGlued'
            else:
                dirPath = f'raw/{self.shape}_modalTest'

        files = os.listdir(dirPath)
        tdmsFiles = list(filter(lambda i: i.split('.')[1] == 'tdms', files))

        # 对每轮模态测试，调用loadTdms函数加载文件，并添加到对应字典
        accelDataSet = {}
        dispDataSet = {}
        for i in range(0, len(tdmsFiles)):
            accelMps, dispMps, accelSet, dispSet = loadTdms(
                f'{dirPath}/{tdmsFiles[i]}')
            accelDataSet[i + 1] = accelSet
            dispDataSet[i + 1] = dispSet

        self.accelDataSet = accelDataSet
        self.dispDataSet = dispDataSet
        self.accelMps = accelMps
        self.dispMps = dispMps



import os
import numpy as np
from wwprocess.dataset.loadTdms import loadTdms

"""
将raw目录下的一个风洞试验对应的一个文件夹封装为一个对象

属性包含：
* 风洞试验下各个测点的加速度和位移的时程数据
方法：
* 在初始化时加载所有风速下的加速度和位移数据
"""


class Wt:  # AeroElasticTest
    """封装了风洞试验数据的对象

    Args:
        shape(str): 截面形状 
        windField(str): 风场条件
        angle(str): 风向角
        noGlued(bool, optional): 是否未贴胶带
        thorough(bool, optional): 是否加密风速
    """

    def __init__(self,
                 shape,  # 形状 ori rect ..
                 windField,  # 风场
                 angle,  # 风向角
                 noGlued=False,  # 是否粘住缝隙
                 thorough=False,  # 是否加密风速
                 testDir='' # 是否输入试验文件夹
                 ):
        # 空值
        self.dispMps = []  # 使用的位移测点
        self.accelMps = []  # 使用的加速度测点
        self.accelDataSet = {}  # 所有风向角的加速度数据
        self.dispDataSet = {}  # 所有风向角的位移数据
        # 描述配置
        # 初始化的值
        self.angle = angle
        self.shape = shape
        self.windField = windField
        self.thorough = thorough
        self.noGlued = noGlued
        self.testDir = testDir
        # 加载风洞试验的平均风速数据
        self.__loadMeanFlow()
        # 加载加速度和位移数据
        self.__loadDataInWindSpeeds()

    def __loadMeanFlow(self):
        """
        加载风速数据
        """
        if self.testDir:
            dirPath = f'raw/{self.testDir}'
        elif self.noGlued:
            if self.thorough:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_noGlued_{self.angle}_thorough'
            else:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_noGlued_{self.angle}'
        else:
            if self.thorough:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_{self.angle}_thorough'
            else:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_{self.angle}'
        meanFlowData = np.loadtxt(
            # TODO
            f'{dirPath}/meanFlow.txt')
        self.meanFlows = meanFlowData  # 平均风速数据
        self.windSpeedSeq = range(1, np.size(meanFlowData) + 1)

    # 计算所有风速的均方根，二级词典，一级为测点，二级为风速
    # 可传入使用的测点编号
    def __loadDataInWindSpeeds(self,
                               ):
        """
        加载所有风速下的数据

        在初始化时被调用，更新加速度和位移属性
        """
        # 处理文件路径
        if self.testDir:
            dirPath = f'raw/{self.testDir}'
        elif self.noGlued:
            if self.thorough:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_noGlued_{self.angle}_thorough'
            else:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_noGlued_{self.angle}'
        else:
            if self.thorough:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_{self.angle}_thorough'
            else:
                dirPath = f'raw/{self.shape}_windTunnelTest_{self.windField}_{self.angle}'

        files = os.listdir(dirPath)
        tdmsFiles = list(filter(lambda i: i.split('.')[1] == 'tdms', files))

        # 在每个风速下，调用loadTdms函数加载文件，并添加到对应字典
        accelDataSet = {}
        dispDataSet = {}
        for i in range(0, len(tdmsFiles)):
            accelMps, dispMps, accelSet, dispSet = loadTdms(f'{dirPath}/{tdmsFiles[i]}')
            accelDataSet[i + 1] = accelSet
            dispDataSet[i + 1] = dispSet

        self.accelDataSet = accelDataSet
        self.dispDataSet = dispDataSet
        self.accelMps = accelMps
        self.dispMps = dispMps



import numpy as np
from wwprocess.calculation.filt import Filt
from wwprocess.calculation.densify import Densify
from wwprocess.calculation.statCalc import Rms


def grepStartX(
    data,  # 纵轴
    rmsVal,  # 均方根值
    error=0.05,  # 和均方根差多少
    numToExam=10  # 向左向右检查多少个
):
    """
    找有效的序列起点，返回起点序列的序号
    """
    effectiveStartX = []
    for ind in range(0, len(data)):
        curError = np.abs(np.abs(data[ind]) - rmsVal) / rmsVal
        # 误差超限则终止
        if curError > error:
            continue
        effective = True
        for i in range(1, numToExam + 1):
            leftInd = ind - i
            rightInd = ind + i
            if leftInd >= 0:  # 左侧序列够用
                compareErrorLeft = np.abs(
                    np.abs(data[leftInd]) - rmsVal) / rmsVal
                if compareErrorLeft < curError:
                    effective = False
                    break
            if rightInd < len(data):  # 右侧序列够用
                compareErrorRight = np.abs(
                    np.abs(data[rightInd]) - rmsVal) / rmsVal
                if compareErrorRight < curError:
                    effective = False
                    break
        if effective == True:
            effectiveStartX.append(ind)
    return effectiveStartX


def getMean(
    data, # 数据序列
    rmsVal, # 使用的均方根值
    lenOfSlice, # 数据长度
):
    """
    对单条数据做平均处理，返回平均后的结果，和使用的数据集长度作为参考
    """
    startX = grepStartX(data, rmsVal)
    subSeqs = []
    dataLen = len(data)
    for ind in startX:
        # 处理不够长的序列
        if ind + lenOfSlice > dataLen:
            continue
        subSeq = np.array(data[ind: ind + lenOfSlice])
        # 处理起始数据点为负值的情况
        if data[ind] < 0:
            subSeq = subSeq * (-1)
        subSeqs.append(subSeq)
    subSeqs = np.array(subSeqs).T
    avgSeq = np.mean(subSeqs, axis=1)
    return avgSeq, len(startX)


def Rdt(wt, modFreq, lenOfSlice = 10000):
    
    Filt(wt, modFreq * 0.75, modFreq * 1.25)
    Densify(wt)
    rms = Rms(wt)

    for ws in wt.windSpeedSeq:
        for mp in wt.accelMps:
            rmsVal = rms.accelDataSet[ws][mp]
            wt.accelDataSet[ws][mp], _ = getMean(wt.accelDataSet[ws][mp], rmsVal, lenOfSlice)
        for mp in wt.dispMps:
            rmsVal = rms.dispDataSet[ws][mp]
            wt.dispDataSet[ws][mp], _ = getMean(wt.dispDataSet[ws][mp], rmsVal, lenOfSlice)
        print(f'wsSeq{ws} finished')



#!/usr/bin/python
"""
此模块中的Schema函数提供带有全局绘图参数的模板

使用方法：
from template import Schema
Schema('single|double')返回plt对象
"""

import matplotlib.pyplot as plt
from cycler import cycler


config = {
    'figure.figsize': (6.27, 4.5),  # 图像尺寸，单位inch，按a4纸，左右边距各2.5cm调整
    # 'figure.figsize': (6.27, 4.7025),  # 图像尺寸，单位inch，按a4纸，左右边距各2.5cm调整
    'figure.constrained_layout.use': True,
    'figure.constrained_layout.h_pad': 0.1,  # 上下留白的比例
    'figure.constrained_layout.w_pad': 0.1,
    'savefig.dpi': 400,
    'savefig.format': 'png',
    'savefig.pad_inches': 0,
    'font.family': 'serif',  # 衬线字体
    'font.serif': ['microsoft yahei'],  # 微软雅黑
    'font.size': 10.5,  # 相当于小四大小
    'mathtext.fontset': 'stix',  # matplotlib渲染数学字体时使用的字体，和times new roman差别不大
    # 'lines.marker': 's',
    'lines.linewidth': 1,
    'lines.markersize': 3,
    'axes.titlesize': 10.5,
    'axes.titlepad': 15,
    'axes.labelsize': 11,
    'axes.labelpad': 7,
    'axes.unicode_minus': False,  # 处理负号，即-号
    'axes.grid': True,
    'axes.grid.which': 'major',
    'xtick.direction': 'in',
    'xtick.minor.visible': True,
    'xtick.labelsize': 8,
    'ytick.direction': 'in',
    'ytick.labelsize': 8,
    'ytick.minor.visible': True,
    'grid.linestyle': '-',
    'grid.linewidth': 0.5,
    'grid.color': '#c1c1c1',
    'legend.fontsize': 9,
    'legend.fancybox': False,
    'legend.edgecolor': '#000',
    'legend.loc': 'best',
}

# 双排图片的排版配置
configDouble = config.copy()
configDouble['figure.figsize'] = (3, 2.25)  # 图像尺寸，单位inch
configDouble['figure.constrained_layout.w_pad'] = 0.02
configDouble['figure.constrained_layout.h_pad'] = 0.05
configDouble['axes.titlesize'] = 9
configDouble['axes.titlepad'] = 7
configDouble['axes.labelsize'] = 10
configDouble['axes.labelpad'] = 5
configDouble['legend.fontsize'] = 7
configDouble['xtick.labelsize'] = 7
configDouble['xtick.minor.visible'] = False
configDouble['ytick.labelsize'] = 7
configDouble['ytick.minor.visible'] = False


def cycleOpt(plt):
    plt.rc('axes', prop_cycle=(
        cycler('color', ['#1f77b4',
                         '#ff7f0e',
                         '#1f77b4',
                         '#ff7f0e'
                         ]) +
        cycler('linestyle', ['-',
                             '-',
                             '--',
                             '--'
                             ])))


def PlotSchema(alignment, useCycle=False):
    """工厂函数，返回plt对象

    Args:
        alignment (str): 'single'单排 | 'double'双排布局
        useCycle (bool, optional): 是否循环颜色和线型. Defaults to False.

    Raises:
        Exception: _description_

    Returns:
        plt: malplotlib.pyplot对象
    """
    if alignment == 'single':
        plt.rcParams.update(config)
    elif alignment == 'double':
        plt.rcParams.update(configDouble)
    else:
        raise Exception('布局参数错误 -> 单列布局：single，双列布局：double')
    # 颜色和线形循环
    if useCycle:
        cycleOpt(plt)
    return plt


# 用于调试全局参数
if __name__ == '__main__':
    import numpy as np

    plt = PlotSchema('single')

    testDataX = np.arange(-10, 10, 0.1)
    testDataY = np.sin(testDataX)

    fig, ax = plt.subplots()

    ax.plot(testDataX, testDataY, label='sine曲线')
    ax.set_title('title 标题 1234 $math^2$')
    ax.set_xlabel('title 标题 1234 $math^2$')
    ax.set_ylabel('title 标题 1234 $math^2$')
    ax.legend()

    plt.show()



#!/usr/bin/python
# coding=utf-8

from setuptools import setup, find_packages
from pathlib import Path

setup(
    name='wwprocess',
    version='0.0.11',
    description=(
        'for processing windworks'
    ),
    long_description=Path('README.md').read_text(encoding='utf8'),
    author='jiang yuanji',
    author_email='jiangyuanji@126.com',
    maintainer='jiang yuanji',
    maintainer_email='jiangyuanji@126.com',
    license='GNU License',
    packages=find_packages(exclude=['tests']), # 列表中传入不查找的目录
    platforms=["all"],
    # url='<项目的网址，我一般都是github的url>',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'matplotlib>=3.5.2',
        'numpy>=1.21.6',
        'scipy>=1.7.3',
        'npTDMS==1.5.0'
    ]
)



# 此脚本用半功率法计算阻尼比
import scipy.signal as sig
import numpy as np

def dumpCalc(
    data,  # 谱序列
    f,  # 频率横轴
    ind,  # 峰值序号
):
    w, wh, l, r = sig.peak_widths(
        data,
        [ind],
        rel_height=1 - 0.7071  # 半功率点的高度
    )
    h = wh[0] # 半功率点高度
    freqL = l[0] * (f[1] - f[0]) # 左频率
    freqR = r[0] * (f[1] - f[0]) # 右频率
    dumpRatio = (freqR - freqL) / (freqR + freqL)
    return freqL, freqR, h, dumpRatio

def dumpCalcWithTimeSeq(ampS, ampE, cyc):
    return np.log(ampS / ampE) / (2 * np.pi * cyc)


if __name__ == '__main__':
    print(np.log(np.e))
    print(np.pi)



from abc import ABC, abstractmethod


class Process(ABC):
    """
    循环调用calc()函数处理数据
    """

    def process(self, exp):
        """
        处理函数，传入试验对象和计算函数，返回相应的结果

        Args:
            exp (Wt/Mt): 试验对象
            calc (function): 计算均方根或均值的函数
        """

        # 定义返回值字典
        results = []  # accel disp

        sigs = [exp.accelDataSet, exp.dispDataSet]
        for sig in sigs:
            dataSet = {}
            for round in sig:  # 对accel disp的每一轮试验
                dataMps = {}
                for mp in sig[round]:  # 对一轮试验的每个测点
                    dataMps[mp] = self.__calc(sig[round][mp])  # 调用计算函数
                dataSet[round] = dataMps
            results.append(dataSet)

        return results[0], results[1]  # 返回加速度和位移

    @abstractmethod
    def __calc(self, mp):
        pass


class ProcessCsd(ABC):
    def processCsd(self, exp, accelMpRef, dispMpRef):
        """
        处理函数，传入试验对象和参考点，返回相应的结果
        """

        # 处理加速度
        accelDataSet = {}
        for round in exp.accelDataSet:  # 对accel disp的每一轮试验
            dataMps = {}
            for mp in exp.accelDataSet[round]:  # 对一轮试验的每个测点
                dataMps[mp] = self.__calcCsd(
                    exp.accelDataSet[round][mp], exp.accelDataSet[round][accelMpRef])  # 调用计算函数
            accelDataSet[round] = dataMps

        # 处理位移
        dispDataSet = {}
        for round in exp.dispDataSet:  # 对accel disp的每一轮试验
            dataMps = {}
            for mp in exp.dispDataSet[round]:  # 对一轮试验的每个测点
                dataMps[mp] = self.__calcCsd(
                    exp.dispDataSet[round][mp], exp.dispDataSet[round][dispMpRef])  # 调用计算函数
            dispDataSet[round] = dataMps

        return accelDataSet, dispDataSet  # 返回加速度和位移

    @abstractmethod
    def __calcCsd(self, mp, mpRef):
        pass



from scipy import signal as sig

# 带通滤波器
def bandpass(data, minFreq, maxFreq, rank=4, fs=500):
    wnMin = 2 * minFreq / fs
    wnMax = 2 * maxFreq / fs
    b, a = sig.butter(rank, [wnMin, wnMax], 'bandpass')
    return sig.filtfilt(b, a, data)


def Filt(wt, minFreq, maxFreq, rank=4, fs=500):
    """
    滤波器

    传入Wt对象，和带通截至频率，对每个数据进行滤波，滤波结果替换原来的数据
    无返回值
    """
    for ws in wt.windSpeedSeq:
        for mp in wt.accelMps:
            wt.accelDataSet[ws][mp] = bandpass(wt.accelDataSet[ws][mp], minFreq, maxFreq, rank, fs)
        for mp in wt.dispMps:
            wt.dispDataSet[ws][mp] = bandpass(wt.dispDataSet[ws][mp], minFreq, maxFreq, rank, fs)
